@@ -5,7 +5,7 @@
 
 package Bio::Minimizer;
 require 5.12.0;
-our $VERSION=0.6;
+our $VERSION=0.8;
 
 use strict;
 use warnings;
@@ -15,28 +15,9 @@ use Carp qw/carp croak/;
 
 use List::MoreUtils qw/uniq/;
 
-# boolean for whether threads are loaded
-our $iThreads; 
-BEGIN{
-  eval{
-    require threads;
-    require threads::shared;
-    $iThreads = 1;
-  };
-  if($@){
-    $iThreads = 0;
-  }
-}
-
-my $startTime = time();
 sub logmsg{
   local $0 = basename $0; 
-  my $tid = 0;
-  if($iThreads){
-    $tid = threads->tid;
-  }
-  my $elapsedTime = time() - $startTime;
-  print STDERR "$0.$tid $elapsedTime @_\n";
+  print STDERR "$0: @_\n";
 }
 
 =pod
@@ -54,6 +35,11 @@ https://academic.oup.com/bioinformatics/article/20/18/3363/202143
     my $kmers     = $minimizer->{kmers};     # hash of minimizer => kmer
     my $minimizers= $minimizer->{minimizers};# hash of minimizer => [kmer1,kmer2,...]
 
+    # hash of minimizer => [start1,start2,...] 
+    # Start coordinates are on the fwd strand even when
+    # matched against the rev strand.
+    my $starts    = $minimizer->{starts}; 
+
     # With more options
     my $minimizer2= Bio::Minimizer->new($sequenceString,{k=>31,l=>21});
 
@@ -66,6 +52,8 @@ Creates a set of minimizers from sequence
 example: Sort a fastq file by minimizer, potentially 
 shrinking gzip size.
 
+This is implemented in this package's scripts/sort*.pl scripts.
+
     use Bio::Minimizer
 
     # Read fastq file via stdin, in this example
@@ -75,9 +63,9 @@ shrinking gzip size.
       chomp($id,$seq,$plus,$qual); 
 
       # minimizer object
-      $MINIMIZER = Bio::Minimizer->new($seq); 
-      # Get smallest minimizer in this entry
-      $minMinimizer = (sort {$a cmp $b} values(%{$$MINIMIZER{minimizers}}))[0]; 
+      $MINIMIZER = Bio::Minimizer->new($seq,{k=>length($seq)}); 
+      # The only minimizer in this entry because k==length(seq)
+      $minMinimizer = (values(%{$$MINIMIZER{minimizers}}))[0]; 
 
       # combine the minimum minimizer with the entry, for
       # sorting later.
@@ -91,16 +79,6 @@ shrinking gzip size.
       print $$e[1];
     } 
 
-=head1 VARIABLES
-
-=over
-
-=item $Bio::Minimizer::iThreads
-
-Boolean describing whether the module instance is using threads
-
-=back
-
 =head1 METHODS
 
 =over
@@ -112,7 +90,7 @@ Boolean describing whether the module instance is using threads
       settings     A hash
         k          Kmer length
         l          Minimizer length (some might call it lmer)
-        numcpus    Number of threads to use. Currently might work against you.
+        numcpus    Number of threads to use. (not used)
 
 =back
 
@@ -132,21 +110,19 @@ sub new{
 
   my $self={
     sequence   => $sequence,
+    revcom     => "",        # revcom of sequence filled in by _minimizers()
     k          => $k,        # kmer length
     l          => $l,        # minimizer length
-    numcpus    => $numcpus,  
+    numcpus    => $numcpus,
     
     # Filled in by _minimizers()
     minimizers => {},        # kmer      => minimizer
     kmers      => {},        # minimizer => [kmer1,kmer2,...]
+    starts     => {},        # minimizer => [start1,start2,...]
   };
 
   bless($self,$class);
 
-  if($numcpus > 1){
-    carp "WARNING: cpus actually slow down this module at this time";
-  }
-
   # Set $$self{minimizers} right away
   $self->_minimizers($sequence);
 
@@ -156,56 +132,54 @@ sub new{
 # Argument: string of nucleotides
 sub _minimizers{
   my($self,$seq) = @_;
-  my %MINIMIZER; 
-  my %KMER;
+
+  my $seqLength = length($seq);
 
   # Also reverse-complement the sequence
   my $revcom = reverse($seq);
   $revcom =~ tr/ATCGatcg/TAGCtagc/;
+  $$self{revcom} = $revcom;
 
   # Length of kmers
   my $k = $$self{k};
 
-  # Split the seq and revcom into numcpus x 2 parts
-  # such that each thread will probably get > 1 subseq
-  # and will hopefully balance out the load.
-  my @subseq;
-  # Ensure some overlap in the subseq lengths by adding in $k.
-  for my $sequence($seq, $revcom){
-    my $sequenceLength = length($sequence);
-    push(@subseq,$sequence);
-  }
-  
-  # If multithreading... 
-  if($iThreads && $$self{numcpus} > 1){
-    my @thr;
-    my $numSubseqPerThread = int(@subseq / $$self{numcpus}) + 1;
-    for(my $i=0;$i<$$self{numcpus};$i++){
-      my @threadSubseq = splice(@subseq, 0, $numSubseqPerThread);
-      $thr[$i] = threads->new(\&minimizerWorker, $self, \@threadSubseq);
-    }
-
-    # merge
-    for my $t(@thr){
-      my $m = $t->join;
-      %MINIMIZER = (%MINIMIZER, %$m);
-    }
-  }
-  # If not multithreading...
-  else {
-    my $minimizers = $self->minimizerWorker(\@subseq);
-    %MINIMIZER = %$minimizers;
-  }
+  # All sequence segments. Probably only seq and revcom.
+  my $fwdMinimizers = $self->minimizerWorker([$seq]);
+  my $revMinimizers = $self->minimizerWorker([$revcom]);
 
+  # Merge minimizer hashes
+  my %MINIMIZER = (%{$$fwdMinimizers{minimizers}}, %{$$revMinimizers{minimizers}});
   $$self{minimizers} = \%MINIMIZER;
 
+  # Merge start site hashes
+  my %START;
+  for my $m(uniq(values(%MINIMIZER))){
+    # Add all start sites for fwd site minimizers
+    for my $pos(@{ $$fwdMinimizers{starts}{$m} }){
+      push(@{ $START{$m} }, $pos);
+    }
+    # recalculate rev minimizer positions
+    for my $pos(@{ $$revMinimizers{starts}{$m} }){
+      my $revPos = $pos;
+      #my $revPos = $seqLength - $pos - $k + 1;
+      push(@{ $START{$m} }, $revPos);
+    }
+
+    $START{$m} = [sort {$a <=> $b} @{$START{$m}}];
+  }
+  $$self{starts} = \%START;
+
+  # Get a hash %KMER of minimizer=>[kmer1,kmer2,...]
+  my %KMER;
   while(my($kmer,$minimizer) = each(%MINIMIZER)){
     push(@{ $KMER{$minimizer} }, $kmer);
   }
+
   # Deduplicate %KMER
   while(my($m, $kmers) = each(%KMER)){
     $kmers = [sort {$a cmp $b} uniq(@$kmers)];
   }
+  #die Dumper $$self{kmers}{ACGTA};
   $$self{kmers} = \%KMER;
 }
 
@@ -213,6 +187,7 @@ sub minimizerWorker{
   my($self, $seqArr) = @_;
 
   my %MINIMIZER; # minimizers that this thread finds
+  my %START;     # minimizer => [start1,start2,...]
 
   # Lengths of kmers and lmers
   my ($k,$l)=($$self{k}, $$self{l}); 
@@ -229,32 +204,49 @@ sub minimizerWorker{
     # all the time between kmers.
     my @lmer;
 
-    for(my $i=0; $i<$numKmers; $i++){
+    for(my $kmerPos=0; $kmerPos<$numKmers; $kmerPos++){
 
-      # The kmer is the subsequence starting at $i, length $k
-      my $kmer=substr($sequence,$i,$k);
+      # The kmer is the subsequence starting at $kmerPos, length $k
+      my $kmer=substr($sequence,$kmerPos,$k);
       
       # Get lmers along the length of the sequence into the @lmer buffer.
-      # The start counter $j how many lmers are already in the buffer.
-      for(my $j=scalar(@lmer); $j < $minimizersPerKmer; $j++){
-        # The lmer will start at $i plus how many lmers are already
+      # The start counter $lmerPos how many lmers are already in the buffer.
+      for(my $lmerPos=scalar(@lmer); $lmerPos < $minimizersPerKmer; $lmerPos++){
+        # The lmer will start at $kmerPos plus how many lmers are already
         # in the buffer @lmer, for length $l.
-        my $lmer = substr($sequence, $i+$j, $l);
-        push(@lmer, $lmer);
+        my $lmer = substr($sequence, $kmerPos+$lmerPos, $l);
+        push(@lmer, [$lmerPos, $lmer]);
       }
 
       # The minimizer is the lowest lmer lexicographically sorted.
-      $MINIMIZER{$kmer} = (sort {$a cmp $b} @lmer)[0];
+      my $minimizerStruct = (sort {$$a[1] cmp $$b[1]} @lmer)[0];
+      $MINIMIZER{$kmer} = $$minimizerStruct[1];
+
+      # Record the start position
+      my $minimizerStart = $$minimizerStruct[0] + $kmerPos;
+      #push(@{ $START{$$minimizerStruct[1]} }, $minimizerStart);
+      $START{$$minimizerStruct[1]}{$minimizerStart}=1;
+
+      #logmsg join("\t", $minimizerStart,$$minimizerStruct[1], $kmer);
 
       # Remove one lmer to reflect the step size of one
       # for the next iteration of the loop.
       my $removedLmer = shift(@lmer);
+      for(@lmer){
+        $$_[0]--; # lmer position decrement
+      }
     }
   }
 
-  # Return kmer=>minimizer
-  return \%MINIMIZER;
+  # Change index to array of unique sites
+  while(my($m, $starts) = each(%START)){
+    #$START{$m} = [sort {$a <=> $b} uniq(@$starts)];
+    $START{$m} = [sort {$a<=>$b} keys(%$starts)];
+  }
+
+  # Return kmer=>minimizer, minimizer=>[start1,start2,...]
+  return {minimizers=>\%MINIMIZER, starts=>\%START};
 }
- 
+
 1;
 
